/*  Titel:  P6 Systemtechnik Low-Cost Absorptionsphotometer
 *  Autor:  Joshua Backlund 
 *  Datum:  07.07.2020
 *  
 *  Für die Funktion und Sicherheit dieses Programms wird keine Garantie gegeben.
 *  Die Variablen müssen je nach verwendeten Komponenten und Funktion angepasst werden.
 */


#include <Wire.h>
#include <Digital_Light_TSL2561.h>
#include <hd44780.h>
#include <hd44780ioClass/hd44780_I2Cexp.h>
#include <SD.h>
#include <SPI.h>
#include "RTClib.h"
#include <avr/interrupt.h>


hd44780_I2Cexp lcd;
File myFile;
RTC_DS1307 RTC;
//DateTime now;


int8_t farbe = 0;
uint32_t Lichtwert = 0;
float Mittelwert = 0;
float ATN = 0;
volatile bool sec = true;
bool first = true;
float refWeissMW = 0;
int cnt = 0;
int cntMW = 0;
char fname[16];
int currentday=-1;


#define Zeit 6      // Zeitintervalle
#define Mittlungszeit 1  //Mittlungszeit in Minuten
#define lpm 1.4     // Luftfluss angeben 

// Initialisierung der Komponenten und des Interrupts
void setup()
{
  DDRD = 0xE0;    // Pin 5,6 und 7 als DO definierten (für RGB)
  PORTD = 0x00;   // Ausgänge noch ausgeschalten
  Wire.begin();
  Serial.begin(9600);
  TSL2561.init();
  lcd.begin(16, 2);
  lcd.home();
  lcd.print("Projekt P6");
  lcd.setCursor(0,1);
  lcd.print("FHNW - ISE");
  delay(4000);
  lcd.clear();
  initSDandRTC();
  createFile();
  Tim(); //Zeit f�r Farbwechsel und Datenrate
}

// Add the main program code into the continuous loop() function
void loop()
{
	if(sec){      // Im Sekundentakt werden die Lichtwerte gemessen und integriert
		sec = false;
		Lichtwert += TSL2561.readVisibleLux();
    Serial.println(TSL2561.readVisibleLux());
    Serial.println(Lichtwert);
    cnt++;
    cntMW++;
	}
	if(cnt>=Zeit){    // Nach der definierten Zeitkonstanten wird der Lichtwert mit der jeweiligen Farbe angezeigt
    cnt = 0;
		if(farbe==0){saveData();}
		myFile.print(","); 
		myFile.print(Lichtwert); 
		if(farbe==4){ myFile.println(); myFile.flush();}  // Nach dem Durchlauf aller Farben speichern der Daten auf die SD-Karte 
    lcd.setCursor(0,0);
    switch(farbe){      // Nach der Anzeige des Lichtwertes auf die nächste Farbe wechseln
      case 0: PORTD = 0x20; lcd.print("Dunkel: "); break; // --> Rot
      case 1: PORTD = 0x40; lcd.print("Rot: "); break; // --> Grün
      case 2: PORTD = 0x80; lcd.print("Gr\365n: "); break; // --> Blau
      case 3: PORTD = 0xE0; lcd.print("Blau: "); break; // --> Weiss
      case 4: PORTD = 0x00; Mittelwert+=Lichtwert;lcd.print("Weiss: "); break; // --> Dunkel
    }
    farbe++;
    if(farbe>4){farbe=0;}	
		lcd.print(Lichtwert);
    lcd.print("      ");
    if(cntMW>=(Mittlungszeit * 60)){
     if(first){refWeissMW=Mittelwert;first=false;}
     cntMW=0;
     lcd.setCursor(0,1);
     lcd.print("eBC: ");
     lcd.print((log(refWeissMW/Mittelwert)-ATN)*7302000.0/lpm);
     lcd.print("[ng/m3]");
     ATN=log(refWeissMW/Mittelwert);
     Mittelwert = 0;
    }
    Lichtwert = 0;
  }
}

// Im Sekundentakt wird der Boolean sec auf ‘true’ gesetzt
ISR(TIMER1_COMPA_vect){
	sec = true;
}

// Jeder Messdurchlauf aller Farben wird zuerst mit dem Zeitstempel versehen (Date, Time) 
void saveData() {
	DateTime now = RTC.now();
	myFile.print('"');
	myFile.print(now.year(), DEC);
	myFile.print("/");
	myFile.print(now.month(), DEC);
	myFile.print("/");
	myFile.print(now.day(), DEC);
	myFile.print(" ");
	myFile.print(now.hour(), DEC);
	myFile.print(":");
	myFile.print(now.minute(), DEC);
	myFile.print(":");
	myFile.print(now.second(), DEC);
	myFile.print('"');
}

// Initialisierung des Interrupts, wird auf eine Sekunde gesetzt
void Tim(void) {
	cli();//stop interrupts
	TCCR1A = 0;// set entire TCCR1A register to 0
	TCCR1B = 0;// same for TCCR1B
	TCNT1 = 0;//initialize counter value to 0
	// set compare match register for 1hz increments
	OCR1A = 15624;// = (16*10^6) / (1*1024) - 1 (must be <65536)
	// turn on CTC mode
	TCCR1B |= (1 << WGM12);
	// Set CS10 and CS12 bits for 1024 prescaler
	TCCR1B |= (1 << CS12) | (1 << CS10);
	// enable timer compare interrupt
	TIMSK1 |= (1 << OCIE1A);
	sei();
}

// Initialisierung der RTC
void initSDandRTC(void){
	    //SD Card initialisation
	    if(SD.begin()){Serial.println("SD Card is ready");}
	    else{ Serial.print("SD Card not ready");}
     
      // connect to RTC
      if (!RTC.begin()) {myFile.println("RTC failed");Serial.println("RTC failed");}
     //{RTC.adjust(DateTime(F(__DATE__), F(__TIME__)));}   // falls RTC vorher nicht Betrieb war, durch PC updaten, sonst auskommentieren
      if (! RTC.isrunning()) {RTC.adjust(DateTime(F(__DATE__), F(__TIME__)));}
}

// Text-Datei wird erstellt, erhält den Namen des aktuellen Datums
void createFile(void) {
      DateTime now = RTC.now();
      //sprintf(fname, "%04i%02i%02i%02i.txt",now.year(),now.month(),now.day(),ncnt);
      sprintf(fname, "%04i%02i%02i.txt",now.year(),now.month(),now.day());
      Serial.println(fname);
	    myFile = SD.open(fname, FILE_WRITE);
	    if(myFile){Serial.println("File ready");}
	    else{Serial.println("File could not be created");}
      // 1. Zeile werden alle Spaltennamen aufgeschrieben
	    myFile.println("Datum/Zeit,Dunkelstrom,Lichtwert Rot,Lichtwert Grün,Lichtwert Blau,Lichtwert Weiss");

}
	
